#!/usr/bin/perl
###############################################################################
# Copyright 2006-2018, Way to the Web Limited
# URL: http://www.configserver.com
# Email: sales@waytotheweb.com
###############################################################################

sub custom_line {
	my $line = shift;
	my $lgfile = shift;

# Do not edit before this point
###############################################################################
#
# Custom regex matching can be added to this file without it being overwritten
# by csf upgrades. The format is slightly different to regex.pm to cater for
# additional parameters. You need to specify the log file that needs to be
# scanned for log line matches in csf.conf under CUSTOMx_LOG. You can scan up
# to 9 custom logs (CUSTOM1_LOG .. CUSTOM9_LOG)
#
# The regex matches in this file will supercede the matches in regex.pm
#
# Example:
#	if (($globlogs{CUSTOM1_LOG}{$lgfile}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ pure-ftpd: \(\?\@(\d+\.\d+\.\d+\.\d+)\) \[WARNING\] Authentication failed for user/)) {
#		return ("Failed myftpmatch login from",$1,"myftpmatch","5","20,21","1","0");
#	}
#
# The return values from this example are as follows:
#
# "Failed myftpmatch login from" = text for custom failure message
# $1 = the offending IP address
# "myftpmatch" = a unique identifier for this custom rule, must be alphanumeric and have no spaces
# "5" = the trigger level for blocking
# "20,21" = the ports to block the IP from in a comma separated list, only used if LF_SELECT enabled. To specify the protocol use 53;udp,53;tcp
# "1" = n/temporary (n = number of seconds to temporarily block) or 1/permanant IP block, only used if LF_TRIGGER is disabled
# "0" = whether to trigger Cloudflare block if CF_ENABLE is set. "0" = disable, "1" = enable

#####   SMTP Exploits   #####
#####   CUSTOM1_LOG = /var/log/mail.log
#
# SMTP Access Denied
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: NOQUEUE: reject: RCPT from \S+\[(\S+)\]: 554 5\.7\.1/)) {
	return ("Access denied from",$1,"smtpaccessdenied","1","","86400","0");
}
# SMTP - Recipient unknown
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: NOQUEUE: reject: RCPT from \S+\[(\S+)\]: 550 5\.1\.1/)) {
	return ("Recipient address rejected from",$1,"smtprecipientunknown","3","","86400","0");
}
# SMTP Hostname unknown
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: NOQUEUE: reject: RCPT from \S+\[(\S+)\]: 450 4\.7\.25 Client host rejected: cannot find your hostname/)) {
	return ("Client host rejected: hostname not found",$1,"smtphostname","3","","86400","0");
}
# SMTP Hostname does not resolve to address
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: warning: hostname \S+ does not resolve to address (\S+)/)) {
	return ("Hostname does not resolve to address",$1,"smtphostnameaddress","3","","86400","0");
}
# SMTP Domain unknown
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: NOQUEUE: reject: RCPT from \S+\[(\S+)\]: 450 4\.1\.8/)) {
	return ("Sender address rejected: domain not found",$1,"smtpdomain","3","","86400","0");
}
# SMTP Lost connection - failure
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: lost connection after \S+ from \S+\[(\S+)\]/)) {
	return ("Lost connection after SMTP failure from",$1,"smtplostconnection","3","","86400","0");
}
# SMTP Disconnect unknown - failure
	if (($lgfile eq $config{CUSTOM1_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ postfix\/smtpd\[\d+\]: disconnect from unknown\[(\S+)\] \S+=\S+/)) {
	return ("Disconnect from unknown after SMTP failure",$1,"smtpdisconnectunknown","3","","86400","0");
}
#
#####   HOST Exploits   #####
#####   CUSTOM2_LOG = /var/log/apache2/keyhelp/access.log
#
# HOST HTTP Request Methods
	if (($lgfile eq $config{CUSTOM2_LOG}) and ($line =~ /^(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:CONNECT|DEBUG|DELETE|MOVE|PUT|TRACE|TRACK|OPTIONS|PATCH|PROPFIND) \S+ HTTP\/\S+" \d+/)) {
	return ("HTTP Request method not allowed on Host from",$1,"hostrequestmethod","1","","86400","0");
}
# HOST Bad Request - TOUGH rule for HTTP/1.0 - Client error responses (400's errors)
	if (($lgfile eq $config{CUSTOM2_LOG}) and ($line =~ /^(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|POST|HEAD) \S+ HTTP\/1\.0.*" (?:400|401|403|404|405|406) \d+ ".*" ".*"/)) {
	return ("Bad request on HTTP/1.0 from",$1,"hostbadrequesthttp1","1","","86400","0");
}
# HOST Very Bad Request - Client error response (400's errors)
	if (($lgfile eq $config{CUSTOM2_LOG}) and ($line =~ /^(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] ".*" (?:408|409|412|413|414|421) \d+ ".*" ".*"/)) {
	return ("Very bad request from",$1,"hostverybadrequest","2","","86400","0");
}
# HOST Login
	if (($lgfile eq $config{CUSTOM2_LOG}) and ($line =~ /^(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] \"GET \/index\.php\?event=1\&username=\S+ HTTP\/\S+\" \d+/)) {
	return ("Too many login attempts on Host from",$1,"hostlogin","3","","86400","0");
}
# HOST Bad Requests - Client error responses (400's errors)
	if (($lgfile eq $config{CUSTOM2_LOG}) and ($line =~ /^(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] ".*" (?:400|403|404|405|406) \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"hostbadrequests","7","","86400","0");
}
#
#####   PHPMYADMIN Login   #####
#####   CUSTOM3_LOG = /var/log/auth.log
#
# phpMyAdmin Auth
	if (($lgfile eq $config{CUSTOM3_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+\s+\S+ phpMyAdmin\[.*\]: user denied: .* \(.*\-denied\) from (\S+)/)) {
	return ("Too many login attempts on phpMyAdmin from",$1,"phpmyadmindenied","3","","86400","0");
}
	if (($lgfile eq $config{CUSTOM3_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+\s+\S+ phpMyAdmin\[.*\]: message repeated \d+ times: \[ user denied: .* \(.*\-denied\) from (\S+)\]/)) {
	return ("Too many login attempts on phpMyAdmin from",$1,"phpmyadmindenied","3","","86400","0");
}
#
#####   WEBMAIL Login   #####
#####   CUSTOM4_LOG = /home/keyhelp/www/rainloop/data/_data_/_default_/logs/webmail-auth.log   #####
#
# Webmail Auth - Rainloop
	if (($lgfile eq $config{CUSTOM4_LOG}) and ($line =~ /^\[\d+\-\d+\-\d+\s+\S+\] Auth failed: ip=(\S+) user=.* host=.* port=.*/)) {
	return ("Too many login attempts on Webmail from",$1,"webmailauth","3","","86400","0");
}
#
#####   BIND DNS Denied   #####
#####   CUSTOM5_LOG = /var/log/syslog
#
# BIND DNS Zone transfer denied
	if (($lgfile eq $config{CUSTOM5_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ named\[\d+\]: client \S+ (\S+)\#\d+ \(\S+\): zone transfer \'\S+\' denied/)) {
	return ("BIND Zone transfer denied from",$1,"dnszonetransferdenied","1","","86400","0");
}
# BIND DNS Recursive queries denied
	if (($lgfile eq $config{CUSTOM5_LOG}) and ($line =~ /^\S+\s+\d+\s+\S+ \S+ named\[\d+\]: client \S+ (\S+)\#\d+ \(\S+\): query \(cache\) \'\S+\/\S+\/\S+\' denied/)) {
	return ("BIND Recursive query denied from",$1,"dnsrecursivequeriesdenied","2","","86400","0");
}
#
#####   OTHER VHOSTS Exploits   #####
#####   CUSTOM6_LOG = /var/log/apache2/keyhelp/other_vhosts_access.log
#
# OTHER VHOSTS HTTP Request Methods
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:POST|CONNECT|DEBUG|DELETE|MOVE|PUT|TRACE|TRACK|OPTIONS|PATCH|PROPFIND) \S+ HTTP\/\S+" \d+/)) {
	return ("Suspicious request originating from",$1,"suspiciousother","1","","86400","0");
}
# OTHER VHOSTS Wordpress wp-config.php
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|HEAD) .*wp-config\.\S+ HTTP\/\S+" \d+/)) {
	return ("Access attempt on WPconfig from",$1,"wpconfigattemptother","1","","86400","0");
}
# OTHER VHOSTS Double slash or http in path
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|HEAD) (?:\/\/|http)\S+ HTTP\/\S+" \d+/)) {
	return ("Suspicious request originating from",$1,"suspiciousother","1","","86400","0");
}
# OTHER VHOSTS CGI not allowed
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|HEAD) .*(?:\.cgi\?|cgi\-bin).* HTTP\/\S+" \d+ ".*" ".*"/)) {
	return ("Suspicious request originating from",$1,"suspiciousother","1","","86400","0");
}
# OTHER VHOSTS Wordpress/Php scripts should only run on SSL
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|HEAD) (?:.*\.php.*|.*wp-admin.*|.*wp-includes.*) HTTP\/\S+" \d+/)) {
	return ("Suspicious requests originating from",$1,"suspiciousphpother","2","","86400","0");
}
# OTHER VHOSTS Very Bad Requests - Client error response (400's errors) extra bad
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] ".*" (?:408|409|412|413|414|421) \d+ ".*" ".*"/)) {
	return ("Very bad requests from",$1,"verybadrequestsother","2","","86400","0");
}
# OTHER VHOSTS Bad Requests - TOUGH rule for HTTP/1.0 - Client error responses (400's errors)
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|HEAD) \S+ HTTP\/1\.0.*" (?:400|401|403|404|405|406) \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"badrequestshttp1other","2","","86400","0");
}
# OTHER VHOSTS Bad Requests - Client error responses (400's errors)
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] ".*" (?:400|401|403|405|406) \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"badrequestsother","4","","86400","0");
}
# OTHER VHOSTS Bad Requests - Too many 404 Not Found
	if (($lgfile eq $config{CUSTOM6_LOG}) and ($line =~ /^\S+\s+(\S+) - - \[\d+\/\S+\/\S+\s+\S+\] ".*" 404 \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"badrequests404other","10","","43200","0");
}
#
#####   DOMAIN Exploits   #####
#####   CUSTOM7_LOG = /var/log/apache2/keyhelp/ipaccess.log
#
# DOMAIN HTTP Request Methods
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] "(?:CONNECT|DEBUG|DELETE|MOVE|PUT|TRACE|TRACK|OPTIONS|PATCH|PROPFIND) \S+ HTTP\/\S+" \d+/)) {
	return ("HTTP Request method not allowed from",$1,"httprequestmethod","1","","86400","0");
}
# DOMAIN Wordpress wp-config.php
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] "\w+ .*wp-config\.\S+ HTTP\/\S+" \d+/)) {
	return ("Access attempt on WPconfig from",$1,"wpconfigattempt","1","","86400","0");
}
# DOMAIN Double slash or http in path
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|POST|HEAD) (?:\/\/|http)\S+ HTTP\/\S+" \d+/)) {
	return ("Suspicious request originating from",$1,"httpsuspicious","1","","86400","0");
}
# DOMAIN CGI not allowed
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|POST|HEAD) .*(?:\.cgi\?|cgi\-bin).* HTTP\/\S+" \d+ ".*" ".*"/)) {
	return ("Suspicious request originating from",$1,"httpsuspicious","1","","86400","0");
}
# DOMAIN Very Bad Requests - Client error responses (400's errors) extra bad
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] ".*" (?:408|409|412|413|414|421) \d+ ".*" ".*"/)) {
	return ("Very bad requests from",$1,"httpverybadrequests","2","","86400","0");
}
# DOMAIN Bad Requests - TOUGH rule for HTTP/1.0 - Client error responses (400's errors)
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|POST|HEAD) \S+ HTTP\/1\.0.*" (?:400|401|403|404|405|406|410) \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"http1badrequests","3","","86400","0");
}
# DOMAIN Wordpress/Php suspicious
# - Remove/modify for generic php app
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] "(?:GET|POST|HEAD) (?:.*\.php.*|.*wp-admin.*|.*wp-includes.*) HTTP\/\S+" (?:400|401|403|404|405|406)/)) {
	return ("Suspicious requests originating from",$1,"suspiciousphp","7","","86400","0");
}
# DOMAIN Bad Requests - Client error responses (400's errors)
# - be aware of false-positives, modify to suit
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] ".*" (?:400|401|403|405|406) \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"httpbadrequests","10","","86400","0");
}
# DOMAIN Bad Requests - Too many 404 Not Found
# - be aware of false-positives, modify to suit
	if (($lgfile eq $config{CUSTOM7_LOG}) and ($line =~ /^(\S+) \S+ - - \[\d+\/\S+\/\S+\s+\S+\] ".*" 404 \d+ ".*" ".*"/)) {
	return ("Too many bad requests from",$1,"httpbadrequests404","25","","21600","0");
}

# If the matches in this file are not syntactically correct for perl then lfd
# will fail with an error. You are responsible for the security of any regex
# expressions you use. Remember that log file spoofing can exploit poorly
# constructed regex's
###############################################################################
# Do not edit beyond this point

	return 0;
}

1;
